package xyz.lychee.lagfixer.map;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.LinearGradientPaint;
import java.awt.RenderingHints;
import java.awt.geom.Path2D;
import java.awt.image.BufferedImage;
import java.util.Collections;
import java.util.LinkedList;
import java.util.concurrent.TimeUnit;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.meta.MapMeta;
import org.bukkit.map.MapCanvas;
import org.bukkit.map.MapCursorCollection;
import org.bukkit.map.MapFont;
import org.bukkit.map.MapPalette;
import org.bukkit.map.MapRenderer;
import org.bukkit.map.MapView;
import org.bukkit.map.MinecraftFont;
import org.bukkit.scheduler.BukkitTask;
import xyz.lychee.lagfixer.LagFixer;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.managers.SupportManager;
import xyz.lychee.lagfixer.objects.AbstractMonitor;
import xyz.lychee.lagfixer.utils.MessageUtils;

public class MapHandler extends MapRenderer {
   private static final int MAP_SIZE = 128;
   private static final int PADDING = 5;
   private final LagFixer plugin;
   private final ItemStack mapItem;
   private final MapView mapView;
   private final BufferedImage image;
   private final LinkedList<Double> values = new LinkedList();
   private final byte[] bytes = new byte[16384];
   private boolean shouldRender = true;
   private BukkitTask task = null;

   public MapHandler(LagFixer plugin) {
      this.plugin = plugin;
      this.mapItem = new ItemStack(Material.FILLED_MAP);
      this.mapView = Bukkit.createMap((World)Bukkit.getWorlds().get(0));
      this.image = new BufferedImage(128, 128, 1);
      Graphics2D g2d = this.image.createGraphics();
      g2d.setColor(Color.GRAY);
      g2d.fillRect(0, 0, 128, 128);
      g2d.dispose();
      ItemMeta meta = this.mapItem.getItemMeta();
      if (meta instanceof MapMeta) {
         ((MapMeta)meta).setMapView(this.mapView);
      }

      meta.setDisplayName(MessageUtils.fixColors((CommandSender)null, "&e⚡ &fServer monitor! &e⚡"));
      this.mapItem.setItemMeta(meta);
      this.mapView.getRenderers().clear();
      this.mapView.addRenderer(this);
   }

   public void load(int interval) {
      double plotWidth = 118.0;
      double plotHeight = 118.0;
      this.task = SupportManager.getInstance().getFork().runTimer(true, () -> {
         AbstractMonitor monitor = SupportManager.getInstance().getMonitor();
         int size = this.values.size();
         if (size >= 20) {
            this.values.removeFirst();
         }

         this.values.addLast(monitor.getMspt());
         if (this.shouldRender) {
            double mspt = monitor.getMspt();
            double tps = monitor.getTps();
            Graphics2D g2d = this.image.createGraphics();
            g2d.setColor(Color.WHITE);
            g2d.fillRect(5, 5, (int)plotWidth, (int)plotHeight);
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            double maxVal = (Double)Collections.max(this.values) * 0.25 + 50.0;
            Path2D.Double path = new Path2D.Double();
            path.moveTo(5.0, Math.max(123.0 - (Double)this.values.get(0) * plotHeight / maxVal, 5.0));

            for(int ix = 0; ix < size; ++ix) {
               double x = 5.0 + (double)ix * plotWidth / (double)(size - 1);
               double y = Math.max(123.0 - (Double)this.values.get(ix) * plotHeight / maxVal, 5.0);
               path.lineTo(x, y);
            }

            path.lineTo(5.0 + plotWidth, 123.0);
            path.lineTo(5.0, 123.0);
            path.closePath();
            float[] fractions = new float[]{0.0F, 0.5F, 1.0F};
            Color[] colors = new Color[]{Color.RED, Color.YELLOW, Color.GREEN};
            LinearGradientPaint gradient = new LinearGradientPaint(0.0F, 5.0F, 0.0F, 123.0F, fractions, colors);
            g2d.setPaint(gradient);
            g2d.fill(path);
            g2d.setStroke(new BasicStroke(2.0F));
            g2d.setColor(new Color(50, 50, 50, 100));

            int ixx;
            for(int i = 1; i < 5; ++i) {
               ixx = (int)(5.0 + (double)i * plotHeight / 5.0);
               g2d.drawLine(123, ixx, 5, ixx);
            }

            g2d.setColor(Color.BLACK);
            this.drawText(10, 10, "Mspt: " + mspt + " Tps: " + tps, Color.BLACK);
            g2d.drawRect(5, 5, (int)plotWidth, (int)plotHeight);
            g2d.dispose();
            int[] pixels = new int[16384];
            this.image.getRGB(0, 0, 128, 128, pixels, 0, 128);

            for(ixx = 0; ixx < pixels.length; ++ixx) {
               this.bytes[ixx] = MapPalette.matchColor(new Color(pixels[ixx], true));
            }

            this.shouldRender = false;
         }
      }, 10L, (long)interval, TimeUnit.SECONDS);
   }

   public void unload() {
      if (this.task != null) {
         this.task.cancel();
      }

   }

   public void render(@NotNull MapView mapView, @NotNull MapCanvas mapCanvas, @NotNull Player player) {
      if (mapView.equals(this.mapView)) {
         PlayerInventory inv = player.getInventory();
         if (inv.getItemInMainHand().getType() == Material.FILLED_MAP || inv.getItemInOffHand().getType() == Material.FILLED_MAP) {
            this.shouldRender = true;
            MapCursorCollection crs = mapCanvas.getCursors();

            while(crs.size() > 0) {
               crs.removeCursor(crs.getCursor(0));
            }

            for(int x = 0; x < 128; ++x) {
               for(int y = 0; y < 128; ++y) {
                  mapCanvas.setPixel(x, y, this.bytes[y * 128 + x]);
               }
            }

         }
      }
   }

   public void drawText(int x, int y, String text, Color color) {
      MinecraftFont font = MinecraftFont.Font;
      int fontHeight = font.getHeight();
      int rgb = color.getRGB();
      char[] var8 = text.toCharArray();
      int var9 = var8.length;

      for(int var10 = 0; var10 < var9; ++var10) {
         char ch = var8[var10];
         MapFont.CharacterSprite sprite = font.getChar(ch);
         if (sprite != null) {
            int spriteWidth = sprite.getWidth();

            for(int r = 0; r < fontHeight; ++r) {
               for(int c = 0; c < spriteWidth; ++c) {
                  if (sprite.get(r, c)) {
                     this.image.setRGB(x + c, y + r, rgb);
                  }
               }
            }

            x += spriteWidth + 1;
         }
      }

   }

   public void giveMap(Player p) {
      p.getInventory().addItem(new ItemStack[]{this.mapItem});
   }
}
